using FarPoint.Win.Spread;
using FarPoint.Win.Spread.CellType;
using GrapeCity.Win.Spread.InputMan.CellType;
using System.ComponentModel;
using System.Diagnostics;

namespace DecimalPointNumberCell
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }


        public Costs myCosts = new Costs();
        public bool IsReading = false;

        private void Form1_Load(object sender, EventArgs e)
        {
            bool oldIsReading = IsReading;
            try
            {
                IsReading = true;

                fpSpread1.Reset();

                fpSpread1.Sheets.Clear();
                Sheet mySheet = new Sheet();
                fpSpread1.Sheets.Add(mySheet);
                fpSpread1.ActiveSheet = mySheet;


                fpSpread1.ClipboardOptions = FarPoint.Win.Spread.ClipboardOptions.ColumnHeaders;
                fpSpread1.ActiveSheet.SelectionPolicy = FarPoint.Win.Spread.Model.SelectionPolicy.MultiRange;


                fpSpread1.EditModePermanent = true;
                fpSpread1.EditModeReplace = true;
                fpSpread1.ButtonDrawMode = ButtonDrawModes.CurrentRow;
                fpSpread1.ScrollBarTrackPolicy = FarPoint.Win.Spread.ScrollBarTrackPolicy.Both;
                fpSpread1.VerticalScrollBarPolicy = ScrollBarPolicy.AsNeeded;
                fpSpread1.HorizontalScrollBarPolicy = ScrollBarPolicy.AsNeeded;


                fpSpread1.AllowColumnMove = true;
                fpSpread1.AllowColumnMoveMultiple = true;
                fpSpread1.RetainSelectionBlock = false;
                fpSpread1.ShowCellErrors = true;
                fpSpread1.TextTipPolicy = FarPoint.Win.Spread.TextTipPolicy.Floating;

                //Add column
                fpSpread1.ActiveSheet.AddColumns(fpSpread1.ActiveSheet.ColumnCount, 1);
                FarPoint.Win.Spread.Column newColumn = fpSpread1.ActiveSheet.Columns[fpSpread1.ActiveSheet.ColumnCount - 1];
                newColumn.Locked = false;
                newColumn.Tag = "test";
                newColumn.Label = "test";

                NumberCellType numCell = new NumberCellType2(fpSpread1, "0.00####");
                numCell.DecimalPlaces = 6;
                numCell.Separator = ",";
                numCell.ShowSeparator = true;
                newColumn.CellType = numCell;
                //fpSpread1.ActiveSheet.AsWorksheet().Cells["A1:A10"].CellType = numCell;
                fpSpread1.ActiveSheet.ColumnHeader.Cells[1, fpSpread1.ActiveSheet.ColumnCount - 1].HorizontalAlignment = CellHorizontalAlignment.Right;

                fpSpread1.ActiveSheet.Rows.Add(0, 1);
                fpSpread1.ActiveSheet.Rows.Add(1, 1);

                myCosts.PropertyChanged += myCosts_PropertyChanged;
                myCosts.ReadEnd += myCosts_ReadEnd;
                fpSpread1.Change += fpSpread1_Change;

                myCosts.ReadRecord();

                this.ActiveControl = textBox2;
                textBox2.Focus();
            }
            finally
            {
                IsReading = oldIsReading;
            }
        }


        private void myCosts_ReadEnd(object sender)
        {
            DisplayCosts();
        }

        private void myCosts_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            DisplayCosts();
        }

        private void fpSpread1_Change(object sender, ChangeEventArgs e)
        {
            textBox1.Text += string.Format("fpSpread1_Change Row:{0}, Column:{1}{2}", e.Row, e.Column, Environment.NewLine);
            object result = fpSpread1.ActiveSheet.Cells[0, 0].Value;
            FarPoint.Win.Spread.CellType.ICellType cellType = fpSpread1.ActiveSheet.GetCellType(e.Row, e.Column);
            myCosts.LastCost = decimal.Parse(((NumberCellType2)cellType).Format(result));
        }

        public void DisplayCosts()
        {
            bool oldIsReading = IsReading;
            var val = fpSpread1.ActiveSheet.Cells[0, 0].Value;
            decimal cellValue;
            try
            {
                IsReading = true;

                FarPoint.Win.Spread.CellType.ICellType cellType = fpSpread1.ActiveSheet.GetCellType(0, 0);
                if (val != null && Decimal.TryParse(fpSpread1.ActiveSheet.Cells[0, 0].Value.ToString(), out cellValue))
                {
                    // do nothing
                }
                else
                {
                    fpSpread1.ActiveSheet.Cells[0, 0].Value = decimal.Parse(((NumberCellType2)cellType).Format(myCosts.LastCost));
                }           
            }
            finally
            {
                IsReading = oldIsReading;
            }
        }
    }

    public class Sheet : FarPoint.Win.Spread.SheetView
    {
        public Sheet() : base()
        {
            RowHeader.ColumnCount = 0;
            RowHeaderVisible = false;
            Columns.Count = 0;
            Rows.Count = 0;
            ColumnHeader.RowCount = 2;
            ColumnHeader.Rows[0].Visible = false;
            ColumnHeader.Rows[1].Height = 45;
        }
    }

    public class NumberCellType2 : NumberCellType
    {
        private FpSpread _spread;

        public NumberCellType2(FpSpread spread, string formatCode)
        {
            _spread = spread;
            NumberFormat = formatCode;
        }

        public string NumberFormat { get; set; }

        public override string Format(object obj)
        {
            GrapeCity.Spreadsheet.NumberFormat numFmt = new GrapeCity.Spreadsheet.NumberFormat(NumberFormat);
            var workbook = _spread.AsWorkbook();
            var formatter = numFmt.GetFormatter(workbook, workbook.WorkbookSet.Culture);
            var result = formatter.Format(obj);
            if (result.Success)
            {
                return result.Text;
            }
            return base.Format(obj);
        }

        public override void PaintCell(Graphics g, Rectangle r, FarPoint.Win.Spread.Appearance appearance, object value, bool isSelected, bool isLocked, float zoomFactor)
        {
            string text = Format(value);
            base.PaintCell(g, r, appearance, text, isSelected, isLocked, zoomFactor);
        }
    }
}